/* 
 * AMD Standard Hot Plug Controller Driver
 *
 * Copyright (C) 2002-2004 Advanced Micro Devices
 *
 * YOUR USE OF THIS CODE IS SUBJECT TO THE TERMS
 * AND CONDITIONS OF THE GNU GENERAL PUBLIC
 * LICENSE FOUND IN THE "GPL.TXT" FILE THAT IS
 * INCLUDED WITH THIS FILE AND POSTED AT
 * http://www.gnu.org/licenses/gpl.html
 *
 * Send feedback to <david.keck@amd.com>
 *
*/

#ifndef _SHPC_DDI_H_
#define _SHPC_DDI_H_

#include "amdshpc.h"

// ****************************************************************************
//
// hp_add_device()
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data (per hardware-instance).
//      driver_context - Caller provided pointer to be returned upon completion.
//      Callback - Caller provided function to be called upon completion of async requests.
//  shpc_instance - Zero-based hardware instance.
//
// ****************************************************************************
long
 hp_add_device(struct shpc_context *shpc_context,
	      void *driver_context,
	      SHPC_ASYNC_CALLBACK Callback, unsigned long shpc_instance);

// ****************************************************************************
//
// hp_Start_device()
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//
// ****************************************************************************
long
 hp_Start_device(struct shpc_context *shpc_context);

// ****************************************************************************
//
// hp_stop_device() 
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//
// Return Value
//  Status returned by any system calls made within hp_stop_device().
//
// ****************************************************************************
long hp_stop_device(struct shpc_context *shpc_context);

// ****************************************************************************
//
// hp_suspend_device()
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//
// Return Value
//  Status returned by any system calls made within hp_suspend_device().
//
// Comments:
//      hp_suspend_device() must be called before transitioning away from PowerDeviceD0.
//
// ****************************************************************************
long hp_suspend_device(struct shpc_context *shpc_context);

// ****************************************************************************
//
// hp_resume_device() 
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//
// Return Value
//  Status returned by any system calls made within hp_resume_device().
//
// Comments:
//      hp_suspend_device() must be called after transitioning back to PowerDeviceD0.
//
// ****************************************************************************
long hp_resume_device(struct shpc_context *shpc_context);

// ****************************************************************************
//
// hp_query_slots() 
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//      SlotConfig - Caller provided storage for slots configuration info.
//
// Return Value
//  Status returned by any system calls made within hp_query_slots().
//
// ****************************************************************************
long hp_query_slots(struct shpc_context *shpc_context,
		   struct slot_config_info *SlotConfig);

// ****************************************************************************
//
// hp_query_slot_status()
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//      slot_id - Zero-based slot number (0..n-1).
//      Query - Pointer to Slot Status Structure
//
// Return Value
//  Status returned by any system calls made within hp_query_slot_status().
//
// ****************************************************************************
long hp_query_slot_status(struct shpc_context *shpc_context, u8 slot_id,
			struct slot_status_info *Query);

// ****************************************************************************
//
// hp_query_slot_psn()
//
// Parameters
//          shpc_context - Caller provided storage for SHPC context data.
//          SlotID - Zero-based slot number (0..n-1).
//          slot_psn - Pointer to Physical Slot Number
//
// Return Value
//  STATUS_SUCCESS, or STATUS_UNSUCCESSFUL for invalid SlotID.
//
// ****************************************************************************
long hp_query_slot_psn(struct shpc_context *shpc_context, unsigned char slot_ID,
		      unsigned long *slot_psn);

// ****************************************************************************
//
// hp_start_async_request()
//
// Parameters
//      shpc_context - Caller provided storage for SHPC context data.
//      slot_id - Zero-based slot number (0..n-1).
//      Request - Async request: Slot "Enable/Disable", AttnLED "Attn/Normal").
//      timeout - For AttnLED "Attn" requests (in seconds)
//      request_context - Caller provided pointer to be returned upon completion.
//
// Comment:
//      For AttnLED "Attn" requests, the completion Callback() function is invoked as soon as the hardware
//      completes (Blink) execution.  When the timeout period expires, the AttnLED is brought back to
//  its "Normal" (On/Off) state, and the Callback() is invoked once again.
//
// ****************************************************************************
long hp_start_async_request(struct shpc_context *shpc_context,
			  u8 slot_id,
			  enum shpc_async_request Request,
			  u32 timeout, void *request_context);

#endif				// _SHPC_DDI_H_
