#ifndef __I386_MMTRACK_H__
#define __I386_MMTRACK_H__

#include <asm/page.h>
#include <asm/atomic.h>
 /*
  * For memory-tracking purposes, if active is true (non-zero), the other
  * elements of the structure are available for use.  Each time mm_track
  * is called, it increments count and sets a bit in the bitvector table.
  * Each bit in the bitvector represents a physical page in memory.
  *
  * This is declared in arch/i386/mm/init.c.
  *
  * The in_use element is used in the code which drives the memory tracking
  * environment.  When tracking is complete, the vector may be freed, but 
  * only after the active flag is set to zero and the in_use count goes to
  * zero.
  *
  * The count element indicates how many pages have been stored in the
  * bitvector.  This is an optimization to avoid counting the bits in the
  * vector between harvest operations.
  */
struct mm_tracker {
	atomic_t active;	// non-zero if this structure in use
	atomic_t count;		// number of pages tracked by mm_track()
	unsigned long * vector;	// bit vector of modified pages
	unsigned long bitcnt;	// number of bits in vector
};
extern struct mm_tracker mm_tracking_struct;

#ifdef CONFIG_X86_PAE
#define PFN_BITS	36
#else /* !CONFIG_X86_PAE */
#define PFN_BITS	32
#endif /* !CONFIG_X86_PAE */

#ifdef CONFIG_MEM_MIRROR
/* The mm_track routine is needed by macros in the pgtable-2level.h
 * and pgtable-3level.h.  The pte manipulation is all longhand below
 * because the required order of header files makes all the useful
 * definitions happen after the following code.
 */
static __inline__ void mm_track(void * val)
{
	pte_t * ptep = (pte_t *) val;

	if (unlikely(atomic_read(&mm_tracking_struct.active))) {
		do {
			unsigned long pfn;

			if ((ptep->pte_low & 0x1) == 0)
				break;

			pfn = pte_val(*ptep) >> PAGE_SHIFT;
			pfn &= ((1LL << (PFN_BITS - PAGE_SHIFT)) - 1);

			if (pfn >= mm_tracking_struct.bitcnt)
				break;

			if (!test_and_set_bit(pfn, mm_tracking_struct.vector))
				atomic_inc(&mm_tracking_struct.count);
		} while (0);
	}
}
#else /* !CONFIG_MEM_MIRROR */
#define mm_track(ptep)
#endif /* !CONFIG_MEM_MIRROR */

#undef MM_PFN_MASK

#endif /* __I386_MMTRACK_H__ */
