#ifndef _LINUX_CKRM_IO_H
#define _LINUX_CKRM_IO_H


#include <linux/fs.h>
#include <linux/blkdev.h>
#include <linux/mempool.h>
#include <linux/ckrm_rc.h>
#include <linux/ckrm_tc.h>


/* root's default sectorate value which
 * also serves as base for absolute shares.
 * Configurable through taskclass' config file. 
 */
struct cki_data {
	/* Protects both */
	spinlock_t cfglock; 
	/* root's absolute shares serve as base for other classes */
	int rootsectorate;
	/* absolute share assigned when relative share is "don't care" */ 
	int minsectorate;
};

struct ps_data {
	struct list_head rr_list;
	struct list_head *dispatch;
	struct list_head *ps_hash;
	struct list_head *prq_hash;

	unsigned int busy_queues;
	unsigned int max_queued;
	atomic_t ref;

	mempool_t *prq_pool;

	request_queue_t *queue;
	char *name;

	struct list_head psdlist;
	/*
	 * tunables
	 */
	unsigned int ps_quantum;
	unsigned int ps_queued;

	/* duration over which sectorates enforced */
	unsigned int ps_epoch;
	/* low-water mark (%) for resuming service of overshare ps_queues */
	unsigned int ps_hmax_pct;
	/* total sectors that queue can sustain */
	unsigned int ps_max_sectorate; 
	/* absolute sectorate when share is a "dontcare" */
	unsigned int ps_min_sectorate;
};

/* For linking all psd's of ps-iosched */
struct psd_list_entry {
	struct list_head psd_list;
	struct ps_data *psd;
};

/* Data for regulating sectors served */
struct ps_rate {
	int nskip;
	unsigned long navsec;
	int timedout;
	atomic_t sectorate;
	u64 sec[2];
};

/* To maintain psrate data structs for each
   request queue managed by ps-iosched */

struct psdrate {
    struct list_head rate_list;
    struct ps_data *psd;
    struct ps_rate psrate;
};

extern struct ckrm_res_ctlr cki_rcbs;
extern struct cki_data ckid;
extern struct ps_rate cki_def_psrate; 

extern struct rw_semaphore psdlistsem;
extern struct list_head ps_psdlist;



int cki_psd_init(struct ps_data *);
int cki_psd_del(struct ps_data *);
void cki_psdlist_add(struct psd_list_entry *);
void cki_psdlist_del(struct ps_data *);

struct ps_rate *cki_tsk_psrate(struct ps_data *, struct task_struct *); 
int cki_recalc_rootrate(void);

#endif 
