/* -*- mode: c; c-basic-offset: 8; -*-
 * vim: noexpandtab sw=8 ts=8 sts=0:
 *
 * uobject.h - generic user object infrastructure.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 021110-1307, USA.
 *
 * Based on uobject.h:
 *      Copyright (c) 2002-2003	Patrick Mochel
 *      Copyright (c) 2002-2003	Open Source Development Labs
 *
 * Copyright (C) 2005 Oracle.  All rights reserved.
 *
 * Please read Documentation/usysfs.txt before using the uobject and
 * usysfs interface, ESPECIALLY the parts about reference counts and
 * object destructors. 
 */

#ifndef _UOBJECT_H_
#define _UOBJECT_H_

#ifdef __KERNEL__

#include <linux/types.h>
#include <linux/list.h>
#include <linux/kref.h>

#define UOBJ_NAME_LEN	20

struct usysfs_attribute;
struct usysfs_attribute_operations;

struct uobject {
	char			*k_name;
	char			name[UOBJ_NAME_LEN];
	struct kref		kref;
	struct list_head	entry;
	struct uobject		*parent;
	struct uset		*uset;
	struct uobj_type	*ktype;
	struct dentry		*dentry;
};

extern int uobject_set_name(struct uobject *, const char *, ...);

static inline char *uobject_name(struct uobject * uobj)
{
	return uobj->k_name;
}

extern void uobject_init(struct uobject *);
extern void uobject_cleanup(struct uobject *);

extern struct uobject * uobject_get(struct uobject *);
extern void uobject_put(struct uobject *);

struct uobj_type {
	void (*release)(struct uobject *);
	struct usysfs_attribute_operations	*sysfs_ops;
	struct usysfs_attribute			**default_attrs;
};


/**
 *	uset - a set of uobjects of a specific type, belonging
 *	to a specific subsystem.
 */

struct uset {
	struct uobject		uobj;
	struct list_head	list;
	struct uset **default_sets;
};


extern void uset_init(struct uset *set);

static inline struct uset *to_uset(struct uobject *uobj)
{
	return uobj ? container_of(uobj,struct uset,uobj) : NULL;
}

static inline struct uset *uset_get(struct uset *set)
{
	return set ? to_uset(uobject_get(&set->uobj)) : NULL;
}

static inline void uset_put(struct uset *set)
{
	uobject_put(&set->uobj);
}

extern struct uobject *uset_find_obj(struct uset *, const char *);


#endif /* __KERNEL__ */
#endif /* _UOBJECT_H_ */
